viSetGet = {};

function viSetGet:generateVehicleCategories()
	VehicleInspector.vehicleCategory = {vehicle={},tool={},misc={}};
	for key, value in pairs(g_storeManager.categories) do
		if value.type == "VEHICLE" then 
			table.insert(VehicleInspector.vehicleCategory.vehicle, string.lower(key));
		elseif value.type == "TOOL" then
			table.insert(VehicleInspector.vehicleCategory.tool, string.lower(key));
		elseif value.type == "MISC" then
			table.insert(VehicleInspector.vehicleCategory.misc, string.lower(key));
		end;
	end;	
	VehicleInspector.vehicleTypeCategory = {		
		train = {"locomotive"};
		tractor = {"tractor", "tractors", "tractorss", "tractorsm", "tractorsl"};
		harvester = {"harvester", "harvesters", "beetvehicles", "potatovehicles", "cottonvehicles", "grapevehicles", "olivevehicles", "sprayervehicles", "sugarcanevehicles", "combinedrivable", "combine"};
		car = {"car", "cars"};
		truck = {"truck", "trucks"};
		other = {"miscvehicles", "woodharvesting", "animalvehicles", "forageharvesters"};
		teleloader = {"loadervehicle", "loadervehicles", "wheelloadervehicles", "teleloadervehicles", "skidsteervehicles", "forklifts", "frontloadervehicles"};
		trailer = {"lowloaders", "seeders", "animaltransport", "loaderwagons", "potatoharvesting", "spayers", "beetharvesting", "sugarcaneharvesting", "fertilizerspreaders", "cottonharvesting", "grapetools", "manurespreaders", "baleloaders", "augerwagons", "planters"};
		tool = {"misc", "doolys", "cutter", "cutters", "plows", "balewrappers", "subsoilers", "movers", "spaders", "wood", "weeders", "tedders", "mulchers", "discharrows"};
	};	
end;

function viSetGet:getVehicleCategory(vehicleCategory, categoryType)
	local categoryName = "unknown";	
	for key, value in pairs(VehicleInspector.vehicleTypeCategory) do
		for k,v in pairs(value) do
			if v.. "_" == vehicleCategory.. "_" then return tostring(key);end;				
		end;
	end;
	if categoryName == "unknown" and categoryType ~= nil and (categoryType == "tool" or categoryType == "vehicle" or categoryType == "misc") then
		for key, value in pairs (VehicleInspector.vehicleCategory[categoryType]) do
			if value.. "_" == vehicleCategory.. "_" then return tostring(categoryType);end;
		end;
		if categoryType == "tool" then categoryName = "tool";end;
	end;
	return categoryName;
end;

function viSetGet:getVehicleCategoryByDesc(typeDesc)
	local categoryName = "unknown";	
	for key, value in pairs(VehicleInspector.vehicleTypeCategory) do
		for k,v in pairs(value) do
			if v.. "_" == typeDesc.. "_" then return tostring(key);end;				
		end;
	end;	
	return categoryName;
end;

--ATTENTION, Mods by onkelyogi no support default
function viSetGet:getVehicleCategoryByXml(self) --Mod friends for VI
	return self.xmlFile:getValue("vehicle.storeData.viCategory#name");	
end;

function viSetGet:getSpeed(vehicle)
	local speed = Utils.getNoNil(vehicle.lastSpeed, 0)*3600;
	if g_gameSettings:getValue('useMiles') then speed = speed*0.621371;end;	
	return string.format("%1.0f", "".. Utils.getNoNil(speed, 0));
end;

function viSetGet:printVehicleType() --for searchInvert
	local vehicle = g_currentMission.controlledVehicle;
	if vehicle ~= nil then
		print("---VehicleInspector: Print Controlled VehicleType---")
		local typeName = Utils.getNoNil(vehicle.typeName, "Unknown"):lower();
		print("**Master Vehicle Type: ".. tostring(typeName))
		if vehicle.getAttachedImplements ~= nil then
			local attachedImplements = vehicle:getAttachedImplements();
			for _, implement in pairs(attachedImplements) do
			   if implement.object ~= nil then
					typeName = Utils.getNoNil(implement.object.typeName, "Unknown"):lower();
					print("**Attached Vehicle Type: ".. tostring(typeName))
			   end;
			end;
		end;		
	end;
end;

function viSetGet:printColorNames() --for colors
	print("---VehicleInspector: Print Color Names---")
	for key, value in pairs(g_currentMission.hl.colorIntern) do
		print("**Color Name: ".. tostring(key))
	end;	
end;

function viSetGet:clickVehicleJoin(args) --click VI default Display	or Moh default Display
	if args == nil or args.rootNode == nil or args.rootNode <= 0 then return;end;
	local vehicle = g_currentMission.nodeToObject[args.rootNode];
	if vehicle == nil then return;end;
	if g_currentMission.controlledVehicle ~= nil and vehicle == g_currentMission.controlledVehicle then return;end;		
	if vehicle.getIsControlled ~= nil and vehicle.getIsControlled(vehicle) then g_currentMission:showBlinkingWarning("Vehicle is not free", 3000);return;end;
	if vehicle:getIsEnterable() then		
		g_currentMission:requestToEnterVehicle(vehicle);
		if g_currentMission.hl.isMouseCursor and VehicleInspector.global.autoMouseOffJoinVehicle then 
			VehicleInspector:get_hl("mouseOnOff", {false,false,g_currentMission.hl.controlMod}, "viSetGet:clickVehicleJoin"); --force mouseOff	
		end;
		return;
	end;
	return;	
end;

function viSetGet:txtColor(state) --for txt colors on/off
	if state then 
		VehicleInspector.showColor = g_currentMission.hl.getTableCopy(VehicleInspector.showColorDefault);
	else		
		for key, value in pairs (VehicleInspector.showColor) do
			VehicleInspector.showColor[key] = "white";
		end;
	end;
end;

--VI LS19 Script, later new
function viSetGet:getAttachedAndSprayerAnd(vehicle, attachedLimit)	
	local attached = 0;
	local attachedSprayerAnd = 0;
	if vehicle.getAttachedImplements ~= nil then 
		local attachTable = vehicle:getAttachedImplements(vehicle);
		if attachTable ~= nil and attachedLimit <= VehicleInspector.global.attachedLimit then		
			for _,attach in pairs(attachTable) do		
				if attach.object ~= nil and attach.object.typeName ~= nil then
					local typeName = Utils.getNoNil(attach.object.typeName, "Unknown"):lower();
					if not string.find(typeName, "implement") and not string.find(typeName, "unknown") then
						attached = attached+1;
					end;					
					if viSetGet:isInvertFill(typeName) then 
						attachedSprayerAnd = attachedSprayerAnd+1;
					end;
					local tempAttached, tempAttachedSprayerAnd = viSetGet:getAttachedAndSprayerAnd(attach.object, attachedLimit+1);
					attached = attached+tempAttached;
					attachedSprayerAnd = attachedSprayerAnd+tempAttachedSprayerAnd;
				end;
			end;
		end;
	end;
	return attached, attachedSprayerAnd;
end;

function viSetGet:isInvertFill(typeName)
	for key,value in pairs(VehicleInspector.global.searchInvert) do
		if value.. "_" == typeName.. "_" then return true;end;				
	end;
	return false;
end;

--VI LS17 Script, later new
function viSetGet:getFillLevelInformationOld(vehicle, fillLevelInformations, isAttached, nr)	
	local attachedNextNr = nr;
	local spec = vehicle.spec_fillUnit;
	if spec ~= nil and spec.fillUnits ~= nil then
		if attachedNextNr <= VehicleInspector.global.attachedLimit then
			for _, fillUnit in pairs(spec.fillUnits) do
				if fillUnit.capacity > 0 and fillUnit.showOnHud then
					local fillType = fillUnit.fillType;

					if fillUnit.fillTypeToDisplay ~= FillType.UNKNOWN then
						fillType = fillUnit.fillTypeToDisplay;
					end;

					local fillLevel = fillUnit.fillLevel;

					if fillUnit.fillLevelToDisplay ~= nil then
						fillLevel = fillUnit.fillLevelToDisplay;
					end;

					local added = false;
					
					local typeName = Utils.getNoNil(vehicle.typeName, "Unknown"):lower();
					
					for _, fillLevelInformation in pairs(fillLevelInformations) do
						if fillLevelInformation.fillType == fillType then
							fillLevelInformation.fillLevel = fillLevelInformation.fillLevel + fillLevel;
							fillLevelInformation.capacity = fillLevelInformation.capacity + fillUnit.capacity;
							added = true;

							break;
						end;
					end;

					if not added then
						table.insert(fillLevelInformations, {
							fillType = fillType;
							fillLevel = fillLevel;
							capacity = fillUnit.capacity;
							isSeedsOrAnd = viSetGet:isInvertFill(typeName);
						});
					end;
				end;
			end;
		end;
	end;
	attachedNextNr = attachedNextNr+1;
    if vehicle.getAttachedImplements ~= nil then
		local attachedImplements = vehicle:getAttachedImplements();
		for _, implement in pairs(attachedImplements) do
		   if implement.object ~= nil then
				fillLevelInformations = viSetGet:getFillLevelInformation(implement.object, fillLevelInformations, true, attachedNextNr);			
		   end;
		end;
	end;
	return fillLevelInformations;
end;

function viSetGet:getFillLevelInformation(vehicle, fillLevelInformations, isAttached, nr, isObject)	
	local attachedNextNr = nr;
	local spec = vehicle.spec_fillUnit;
	local foundFillType = false;
	local typeName = Utils.getNoNil(vehicle.typeName, "Unknown"):lower();	
	if spec ~= nil and spec.fillUnits ~= nil then
		if attachedNextNr <= VehicleInspector.global.attachedLimit then			
			for i = 1, #spec.fillUnits do
				local fillUnit = spec.fillUnits[i];
				if fillUnit.capacity > 0 and fillUnit.showOnHud then
					local fillType = fillUnit.fillType;
					if fillType == FillType.UNKNOWN and table.size(fillUnit.supportedFillTypes) == 1 then
						fillType = next(fillUnit.supportedFillTypes);
					end;
					if fillUnit.fillTypeToDisplay ~= FillType.UNKNOWN then
						fillType = fillUnit.fillTypeToDisplay;
					end;
					local fillLevel = fillUnit.fillLevel;
					if fillUnit.fillLevelToDisplay ~= nil then
						fillLevel = fillUnit.fillLevelToDisplay;
					end;
					local capacity = fillUnit.capacity;
					if fillUnit.parentUnitOnHud ~= nil then
						if fillType == FillType.UNKNOWN then
							fillType = spec.fillUnits[fillUnit.parentUnitOnHud].fillType;
						end;
						capacity = 0;
					elseif fillUnit.childUnitOnHud ~= nil and fillType == FillType.UNKNOWN then
						fillType = spec.fillUnits[fillUnit.childUnitOnHud].fillType;
					end;
					local maxReached = not fillUnit.ignoreFillLimit and g_currentMission.missionInfo.trailerFillLimit and vehicle.getMaxComponentMassReached ~= nil and vehicle:getMaxComponentMassReached();
					fillLevelInformations = viSetGet:addFillLevel(fillLevelInformations, typeName, fillType, fillLevel, capacity, isObject or false, fillLevel > 0 and fillUnit.uiPrecision or 0, maxReached);
					foundFillType = true;
				end;
			end;
		end;
	end;
	if (isObject == nil or not isObject) then
		if vehicle.spec_tensionBelts ~= nil then
			fillLevelInformations = viSetGet:getOtherFillLevelInformation(vehicle, fillLevelInformations, "tensionbelts");
		end;
		if vehicle.spec_treePlanter ~= nil then
			fillLevelInformations = viSetGet:getOtherFillLevelInformation(vehicle, fillLevelInformations, "treeplanter");
		end;
		if vehicle.spec_dynamicMountAttacher ~= nil then
			fillLevelInformations = viSetGet:getOtherFillLevelInformation(vehicle, fillLevelInformations, "mountattacher");
		end;
	end;
	attachedNextNr = attachedNextNr+1;
    if vehicle.getAttachedImplements ~= nil then
		local attachedImplements = vehicle:getAttachedImplements();
		for _, implement in pairs(attachedImplements) do
		   if implement.object ~= nil then
				fillLevelInformations = viSetGet:getFillLevelInformation(implement.object, fillLevelInformations, true, attachedNextNr);			
		   end;
		end;
	end;
	return fillLevelInformations;
end;

function viSetGet:addFillLevel(fillLevelInformations, typeName, fillType, fillLevel, capacity, isObject, precision, maxReached)
	local added = false;
	for j = 1, #fillLevelInformations do
		local fillLevelInformation = fillLevelInformations[j];
		if fillLevelInformation.fillType == fillType then
			fillLevelInformation.fillLevel = fillLevelInformation.fillLevel + fillLevel;
			fillLevelInformation.capacity = fillLevelInformation.capacity + capacity;
			fillLevelInformation.precision = precision;
			fillLevelInformation.maxReached = maxReached;			
			if isObject and fillLevelInformation.numObject == nil then fillLevelInformation.numObject = 1;elseif isObject then fillLevelInformation.numObject = fillLevelInformation.numObject+1;end;
			added = true;
			break;
		end;
	end;
	if not added then
		if isObject then isObject = 1;else isObject = nil;end;
		table.insert(fillLevelInformations, {
			fillType = fillType;
			fillLevel = fillLevel;
			capacity = capacity;
			precision = precision;			
			maxReached = maxReached;
			numObject = isObject;
			isSeedsOrAnd = viSetGet:isInvertFill(typeName);
		});		 
	end;
	return fillLevelInformations;
end;

function viSetGet:getOtherFillLevelInformation(vehicle, fillLevelInformations, otherType)	
	if otherType == "tensionbelts" and vehicle.spec_tensionBelts.hasTensionBelts then
		local spec = vehicle.spec_tensionBelts;
		for _, objectData in pairs(spec.objectsToJoint) do
			local object = objectData.object;
			if object ~= nil then
				fillLevelInformations = viSetGet:getFillLevelInformation(object, fillLevelInformations, true, 0, true);				
			end;
		end;
	elseif otherType == "treeplanter" and vehicle.spec_treePlanter.mountedSaplingPallet then
		fillLevelInformations = viSetGet:getFillLevelInformation(vehicle.spec_treePlanter.mountedSaplingPallet, fillLevelInformations, true, 0, false);
	elseif otherType == "mountattacher" and vehicle.spec_dynamicMountAttacher.dynamicMountedObjects then
		local spec = vehicle.spec_dynamicMountAttacher;
		for object, _ in pairs(spec.dynamicMountedObjects) do
			if object ~= nil then
				fillLevelInformations = viSetGet:getFillLevelInformation(object, fillLevelInformations, true, 0, true);
			end;
		end;		
	end;
	return fillLevelInformations;
end;

function viSetGet:getPlayerName(vehicle)
	--if old then
		--if g_currentMission.controlledVehicle ~= nil and g_currentMission.controlledVehicle == vehicle then --later search PlayerName at Vehicle !is Bug!	
		--	return g_gameSettings.onlinePresenceName, false;
		--else
			if vehicle.spec_enterable ~= nil and vehicle.spec_enterable.controllerUserId ~= nil then
				local user = g_currentMission.userManager:getUserByUserId(vehicle.spec_enterable.controllerUserId);
				if user ~= nil then			
					local userName = Utils.getNoNil(user:getNickname(),VehicleInspector.txt.isPlayer);
					return userName, userName ~= g_gameSettings.onlinePresenceName;
				end;
			end;		
			return VehicleInspector.txt.isPlayer, true; --error
		--end;	
		--return VehicleInspector.txt.isPlayer, true; --error
	--end;
	--return VehicleInspector.txt.isPlayer, true; --error	
	--return vehicle.getControllerName(vehicle), g_gameSettings.nickname ~= vehicle.getControllerName(vehicle);
end;

function viSetGet:getFarmContractor(playerFarmId, vehicleFarmId)
	if not g_currentMission.missionDynamicInfo.isMultiplayer then return true;end;
	local vehicleFarm = g_farmManager:getFarmById(vehicleFarmId);
	if vehicleFarm ~= nil then
		return vehicleFarm:getIsContractingFor(playerFarmId); 
	end;
	return false;
end;

function viSetGet:getFarmIdTables(farmTables, playerFarmId)
	local farms = g_farmManager:getFarms();
	if farms ~= nil then
		for _, farm in pairs(farms) do
			farmTables[farm.farmId] = {playerPos=0,helperPos=0,name=tostring(farm.name)};			
		end;
	end;
	if #farmTables <=0 then farmTables[playerFarmId] = {playerPos=0,helperPos=0,name="MyFarm"};end;
	return farmTables;
end;

function viSetGet:getCollision(vehicle)	
	if not VehicleInspector.seeWarning.isBlocking then return false;end;
	--lastHasCollision
	--if g_server ~= nil and g_client ~= nil and g_dedicatedServerInfo ~= nil then return false;end;
	if vehicle.spec_aiFieldWorker ~= nil and vehicle.spec_aiFieldWorker.isActive then
		--if vehicle:getLastSpeed() < 0.5 then				
			if vehicle.spec_aiFieldWorker.didNotMoveTimer < 0 and not vehicle.spec_aiFieldWorker.isTurning then return true;end;			
		--end;
	end;
	return false;
end;

--VI LS17 Script, later new
function viSetGet:setVehicleName(vehicle, attachable)
	local vehicleName = "Unknown";
	local category = "Unknown";
	function setAlternativeName()
		if vehicle.baleWidth ~= nil then
			vehicle.viVehicleName = "Bale";
			vehicleName = "Bale";
		elseif vehicle.fillablePalletDirtyFlag ~= nil then
			vehicle.viVehicleName = "Pal.";
			vehicleName = "Pal.";
		end;
	end;
		
	if vehicle.configFileName == nil then 
		setAlternativeName();
		vehicle.viVehicleName = vehicleName;
		return;
	end;
	if vehicle.viVehicleName ~= nil then return;end;		
	
	vehicleName = tostring(vehicle.getFullName(vehicle));
	
	if vehicle.configFileName ~= nil then
		if string.find(tostring(vehicle.configFileName), "locomotive") then
			vehicleName = "Train";		
			if not attachable then vehicle.typeDesc = vehicleName;end;
		elseif string.find(tostring(vehicle.configFileName), "horse") then
			vehicleName = "Horse";
		elseif string.find(tostring(vehicle.configFileName), "stationCrane") then
			vehicleName = "Crane";
			if not attachable then vehicle.typeDesc = vehicleName;end;
		elseif string.find(tostring(vehicle.configFileName), "wagon") then
			vehicleName = "WagonFill";
			if string.find(tostring(vehicle.configFileName:lower()), "timber") then
				vehicleName = "WagonTimber";
			end;
			if attachable then vehicle.typeDesc = vehicleName;end;
		elseif string.find(tostring(vehicle.configFileName), "ka26") then
			vehicleName = "Heli";
			if not attachable then vehicle.typeDesc = vehicleName;end;
		elseif vehicle.time ~= nil and vehicle.time > 0 then
			vehicleName = vehicleName.. "-Missions Vehicle";
		end;
	end;
	if vehicleName == "Unknown" then setAlternativeName();end;
	--tableVehicle.name = vehicleName;	
	vehicle.viVehicleName = vehicleName;
end;

function viSetGet:getWater(vehicle)
	if not VehicleInspector.seeWarning.isWater then return false;end;
	return vehicle.isInWater or vehicle.showTailwaterDepthWarning;
end;

function viSetGet:getDamage(vehicle)
	if not VehicleInspector.seeWarning.isDamage or vehicle.getDamageAmount == nil then return false;end;
	local damageLevel = math.min(1, 1 - vehicle:getDamageAmount());
	if damageLevel == nil then return false;end;
	return vehicle.isBroken or damageLevel < 0.1;	
end;

function viSetGet:getFuel(vehicle)
	if not VehicleInspector.seeWarning.isFuelEmpty then return false;end;
	if vehicle.spec_motorized == nil or vehicle.getConsumerFillUnitIndex == nil then return false;end;	
	local fuelLevel, capacity = viSetGet:getVehicleFuelLevelAndCapacity(vehicle);
	if capacity == nil or capacity <= 0 or fuelLevel == nil then return false;end;	
	return fuelLevel < 1;
end;

function viSetGet:getVehicleFuelLevelAndCapacity(vehicle)
	local fuelType = FillType.DIESEL
	local fillUnitIndex = vehicle:getConsumerFillUnitIndex(fuelType);
	if fillUnitIndex == nil then
		fuelType = FillType.ELECTRICCHARGE;
		fillUnitIndex = vehicle:getConsumerFillUnitIndex(fuelType);
		if fillUnitIndex == nil then
			fuelType = FillType.METHANE;
			fillUnitIndex = vehicle:getConsumerFillUnitIndex(fuelType);
		end;
	end;
	local fuelLevel = vehicle:getFillUnitFillLevel(fillUnitIndex);
	local capacity = vehicle:getFillUnitCapacity(fillUnitIndex);
	return fuelLevel, capacity, fuelType;
end;

function viSetGet:getIsOnField(vehicle)
	local fieldNumber = 0;
	local isField = false;
	local wx, wy, wz = 0,0,0;
	function getIsOnField()
		if vehicle.components == nil then return false;end;
		for _, component in pairs(vehicle.components) do
			wx, wy, wz = localToWorld(component.node, getCenterOfMass(component.node));
			local h = getTerrainHeightAtWorldPos(g_currentMission.terrainRootNode, wx, wy, wz);
			if wy < h - 1 then
				break;
			end;
			local isOnField, _ = FSDensityMapUtil.getFieldDataAtWorldPosition(wx, wy, wz);
			if isOnField then
				isField = true;
				return true;
			end;
		end;
		return false;
	end;
	if getIsOnField() and VehicleInspector.seeTxt.fieldNumber and ( (VehicleInspector.global.seeFieldNumberMP and g_currentMission.missionDynamicInfo.isMultiplayer) or (not g_currentMission.missionDynamicInfo.isMultiplayer) ) then		
		local farmlandId = g_farmlandManager:getFarmlandIdAtWorldPosition(wx, wz);	
		if farmlandId ~= nil then
			local foundField = false;
			for f=1, #g_fieldManager.fields do
				if foundField then break;end;
				local field = g_fieldManager.fields[f];
				if field ~= nil and field.farmland ~= nil and field.farmland.id == farmlandId then
					local fieldId = field.fieldId;
					for a=1, #field.setFieldStatusPartitions do --field.getFieldStatusPartitions
						local b = field.setFieldStatusPartitions[a];						
						local x, z, wX, wZ, hX, hZ = b.x0, b.z0, b.widthX, b.widthZ, b.heightX, b.heightZ;
						local distanceMax = math.max(wX,wZ,hX,hZ);
						local distance = MathUtil.vector2Length(wx-x,wz-z);
						if distance <= distanceMax then
							fieldNumber = fieldId;
							foundField = true;
							break;
						end;
					end;	
				end;
			end;			
		end;		
	end;	
	return isField, fieldNumber;
end;

function viSetGet:getFieldData(data)
	if data == nil then return false;end;
	--FSDensityMapUtil.getFieldStatusAsync(startWorldX, startWorldZ, widthWorldX, widthWorldZ, heightWorldX, heightWorldZ, viSetGet.getFieldData, viSetGet.getFieldData);
end;

--VI LS19 Script, later new
function viSetGet:getIsOnFieldOld(vehicle)
	--lastHasNoField
    local fieldNumber = 0;
	local isField = false;
	local wx, wy, wz = 0,0,0;
	function getIsOnField()
		local densityBits = 0;
		for _,component in pairs(vehicle.components) do
			wx, wy, wz = getWorldTranslation(component.node);
			local h = getTerrainHeightAtWorldPos(g_currentMission.terrainRootNode, wx, wy, wz);
			if h-1 > wy then 
				break;
			end
			local bits = getDensityAtWorldPos(g_currentMission.terrainDetailId, wx, wy, wz);
			densityBits = bitOR(densityBits, bits);
			if densityBits ~= 0 then
				isField = true;
				return true;
			end
		end
		return false;
	end
	getIsOnField();
	--if getIsOnField() and VehicleInspector.seeWhat.fieldNumber then		
	--	local foundField = false;
	--	for fieldNum,fieldDef in ipairs(g_fieldManager.fields) do		
	--		if foundField then break;end;
	--		for a=1, #fieldDef.setFieldStatusPartitions do
	--			local b = fieldDef.setFieldStatusPartitions[a];
	--			local x, z, wX, wZ, hX, hZ = b.x0, b.z0, b.widthX, b.widthZ, b.heightX, b.heightZ;
	--			local distanceMax = math.max(wX,wZ,hX,hZ);
	--			local distance = MathUtil.vector2Length(wx-x,wz-z);
	--			local fruitDesc = g_currentMission.fruitTypeManager.fruitTypes[fieldDef.fruitType];				
				--renderText(0.5, 0.96, 0.010, "fruitDesc_ ".. tostring(fruitDesc.name));
	--			local noFruit = fruitDesc ~= nil and fruitDesc.name == "UNKNOWN" --or fruitDesc.name == "GRASS" or fruitDesc.name == "DRYGRASS" or fruitDesc.name == "WEED"
	--			if not noFruit and distance <= distanceMax then 
	--				fieldNumber = fieldDef.fieldId;foundField=true;
					--print("distance...".. tostring(distance).. " - maxDistance...".. tostring(distanceMax))
	--				break;
	--			end;				
	--		end;			
	--	end;
	--end;
	return isField, fieldNumber;	
end;

--VI LS19 Script, later new
function viSetGet:getInGameDisplay()
	local foundHelp = false;
	local foundMap = false;
	if g_currentMission.hud.ingameMap.state ~= nil then
		VehicleInspector.inGameMapMenu = g_currentMission.hud.ingameMap;foundMap=true;		
	end;
	if g_currentMission.hud.inputHelp.controlsLabelText ~= nil and g_currentMission.hud.inputHelp.numUsedEntries ~= nil then
		VehicleInspector.helpMenu = g_currentMission.hud.inputHelp;foundHelp=true;		
	end;
	if foundHelp and foundMap then return;end;
	for key, value in pairs(g_currentMission.hud.displayComponents) do
		if not foundHelp and g_currentMission.hud.displayComponents[key].controlsLabelText ~= nil and g_currentMission.hud.displayComponents[key].numUsedEntries ~= nil then
			VehicleInspector.helpMenu = g_currentMission.hud.displayComponents[key];
			foundHelp = true;
		end;
		if not foundMap and g_currentMission.hud.displayComponents[key].state ~= nil then
			VehicleInspector.inGameMapMenu = g_currentMission.hud.displayComponents[key];
			foundMap = true;
		end;
		if foundMap and foundHelp then break;end;
	end;
end;

function viSetGet:helpMenue()
	local state = false;
	local helpMenu = g_gameSettings:getValue("showHelpMenu");
	if helpMenu and VehicleInspector.showDraw.helpMenuOn then state = true;end;	
	if not state and not helpMenu and VehicleInspector.showDraw.helpMenuOff then state = true;end;
	if state then state = viSetGet:refreshPosX_Y(helpMenu);end;
	return state;
end;

function viSetGet:inGameMapMenu()	
	local inGameMapState = VehicleInspector:get_hl("getIngameMapState", {}, "viSetGet:inGameMapMenu");
	if (inGameMapState == 2 or inGameMapState == 3) and VehicleInspector.showDraw.inGameMapOn then return true;end;
	if (inGameMapState == 0 or inGameMapState == 1) and VehicleInspector.showDraw.inGameMapOff then return true;end;
	return false;
end;

function viSetGet:refreshPosX_Y(helpMenu)
	if VehicleInspector.helpMenu == nil then return false;end;
	if helpMenu then
		local posX, posY = InputHelpDisplay.getBackgroundPosition();
		VehicleInspector.global.posY[VehicleInspector.global.viewModus] = VehicleInspector.helpMenu.overlay.y-VehicleInspector.helpMenu.entryHeight+(VehicleInspector.global.posY.otherMod)+(VehicleInspector.global.fixPosY.onF1);
		VehicleInspector.global.posX[VehicleInspector.global.viewModus] = posX+(VehicleInspector.global.posX.otherMod)+(VehicleInspector.global.fixPosX.onF1);
	else
		local posX, posY = viSetGet:getVehicleSchemaPosition(false);
		if g_currentMission.controlledVehicle ~= nil then
			VehicleInspector.global.posY[VehicleInspector.global.viewModus] = posY+(VehicleInspector.global.posY.otherMod)+(VehicleInspector.global.fixPosY.onSchema);
			VehicleInspector.global.posX[VehicleInspector.global.viewModus] = posX+(VehicleInspector.global.posX.otherMod)+(VehicleInspector.global.fixPosX.onSchema);
		else
			VehicleInspector.global.posY[VehicleInspector.global.viewModus] = VehicleInspector.global.posY.default+(VehicleInspector.global.posY.otherMod)+(VehicleInspector.global.fixPosY.offF1);
			VehicleInspector.global.posX[VehicleInspector.global.viewModus] = VehicleInspector.global.posX.default+(VehicleInspector.global.posX.otherMod)+(VehicleInspector.global.fixPosX.offF1);
		end;
	end;
	return true;
end;

function viSetGet:getVehicleSchemaPosition(isDocked)
	local uiScale = g_gameSettings:getValue("uiScale")
	local width, height = getNormalizedScreenValues(unpack(VehicleSchemaDisplay.SIZE.SELF));
	local posX = g_safeFrameOffsetX;
	local posY = 1 - g_safeFrameOffsetY - height * uiScale;
	if isDocked then --helpMenue Open and controlledVehicle
		local offX, offY = getNormalizedScreenValues(unpack(VehicleSchemaDisplay.POSITION.SELF_DOCKED));
		posX = posX + (offX - width) * uiScale;
		posY = posY + offY * uiScale;
	end;
	return posX, posY;
end;

local oldVehicleLoad = Vehicle.load;
Vehicle.load = function (self, vehicleData, asyncCallbackFunction, asyncCallbackObject, asyncCallbackArguments)
	local isOkay, result = pcall(oldVehicleLoad, self, vehicleData, asyncCallbackFunction, asyncCallbackObject, asyncCallbackArguments);
	if isOkay and self ~= nil then
		local isRegister = false;
		if Vehicle.xmlSchema ~= nil then Vehicle.xmlSchema:register(XMLValueType.STRING, "vehicle.storeData.viCategory#name", "VehicleInspector");isRegister = true;end;
		local item = g_storeManager:getItemByXMLFilename(self.configFileName);
		local typeName = Utils.getNoNil(self.typeName, "unknown");
		local typeVehicle = Utils.getNoNil(self.type, nil);			
		local categoryByModFriends = nil;
		if isRegister and self.xmlFile ~= nil then categoryByModFriends = viSetGet:getVehicleCategoryByXml(self);end;
		if item ~= nil and categoryByModFriends == nil then				
			local categoryName = tostring(string.lower(item.categoryName));
			--if categoryName ~= "onkelyogi" then 
				local category = g_storeManager:getCategoryByName(categoryName)
				if category == nil then category = g_storeManager:getCategoryByName("misc");end;				
				local foundCategoryName = "unknown";
				local foundTypeDescName = "unknown";			
				if typeVehicle ~= nil and type(typeVehicle) == "table" and typeVehicle.name ~= nil then 
					foundTypeDescName = viSetGet:getVehicleCategoryByDesc(tostring(string.lower(typeVehicle.name)));
				end;
				if typeName == "locomotive" then
					foundCategoryName = "train";
				else
					foundCategoryName = viSetGet:getVehicleCategory(categoryName, tostring(string.lower(category.type)));
					if foundCategoryName == nil or string.len(foundCategoryName) <= 0 then foundCategoryName = "unknown";end;					
				end;			
				if foundCategoryName == "trailer" then
					if self.spec_fillUnit == nil then 
						foundCategoryName = "tool";
					end;
				elseif foundCategoryName == "vehicle" then					
					if foundTypeDescName ~= "unknown" and foundTypeDescName ~= "tool" then
						foundCategoryName = foundTypeDescName;
					else
						foundCategoryName = "other";
					end;
				elseif categoryName == "misc" or foundCategoryName == "tool" or foundCategoryName == "misc" then
					if self.spec_enterable ~= nil then 
						if foundTypeDescName ~= "unknown" and foundTypeDescName ~= "tool" then
							foundCategoryName = foundTypeDescName;
						else
							foundCategoryName = "other";
						end;
					elseif self.spec_fillUnit ~= nil or self.spec_fillVolume ~= nil then
						foundCategoryName = "trailer";					
					else
						foundCategoryName = "tool";
					end;					
				end;				
				self.viCategoryName = foundCategoryName;
			--else
			--	self.viCategoryName = "nosupport";
			--end;
		else
			if categoryByModFriends ~= nil then					
				self.viCategoryName = tostring(categoryByModFriends):lower();
			else
				self.viCategoryName = "unknown";
			end;
		end;		
		return result;
	end;
	return false;
end;




















